#!/usr/bin/perl
#
# This program first reads in the automatic metrics' scores.  It then
# reads in the human judgments and for each item, it checks the correspondence.

# English metrics
@metrics = ( "HUMAN_RANK", "AMBER", "AMBER_nl", "AMBER_ti", "BLEU", "dfki_parseconf", "F15", "F15g3", "meteor-1.3-adq", "meteor-1.3-rank", "mp4ibm1", "mpF", "MTeRater", "MTeRater-Plus", "ROSE", "TER", "tesla-b", "tesla-f", "tesla-m", "TINE-srl_match", "wmpF" );

@errorMetrics = ();
$errorMetrics{"mp4ibm1"} = 1;
$errorMetrics{"MTeRater"} = 1;
$errorMetrics{"MTeRater-Plus"} = 1;
$errorMetrics{"TER"} = 1;

# this variable should be set to 0 if you want to limit to only 
# those judgments that we have identical judgments for.
$analyzeAll = 1;


open(JUDGMENTS, $ARGV[0]) or die("Couldn't open $ARGV[0]\n");
$line = <JUDGMENTS>;

# FILE FORMAT FOR THE WMT11 human judgments
#($srclang, $trglang, $srcIndex, $documentId, $segmentId, $judgeId, $system1Number, $system1Id, $system2Number, $system2Id, $system3Number, $system3Id, $system4Number, $system4Id, $system5Number, $system5Id, $system1rank, $system2rank, $system3rank, $system4rank, $system5rank)
#  $srclang = $fields[0];
#  $trglang = $fields[1];
#  $srcIndex = $fields[2];
#  $documentId = $fields[3];
#  $segmentId = $fields[4];
#  $judgeId = $fields[5];
#  $system1Number = $fields[6];
#  $system1Id = $fields[7];
#  $system2Number = $fields[8];
#  $system2Id = $fields[9];
#  $system3Number = $fields[10];
#  $system3Id = $fields[11];
#  $system4Number = $fields[12];
#  $system4Id = $fields[13];
#  $system5Number = $fields[14];
#  $system5Id = $fields[15];
#  $system1rank = $fields[16];
#  $system2rank = $fields[17];
#  $system3rank = $fields[18];
#  $system4rank = $fields[19];
#  $system5rank = $fields[20];   

while($line = <JUDGMENTS>) {
    chomp $line;
    @fields = split(/,/, $line);
    $sourceLang = $fields[0];
    $targetLang = $fields[1];
    $segID = $fields[2]; 
    ($langPair, $testSet) = split(/\./, $fields[3]);
    $judgmentID = $fields[4];
    $annotatorID = $fields[5];
    $testSet = "newssyscombtest2011";
    $taskName = "WMT11 " . $sourceLang . "-" . $targetLang . " " . $testSet; 
    $testType = "HUMAN_RANK"; 


    @systems = ();
    for($i = 7; $i < 16; $i+=2) {
	$system = $fields[$i];
	push(@systems, $system);
    }

    @scores = ();
    for($i = 16; $i < @fields; $i++) {
	$score = $fields[$i];
	# -1 indicates no system.
	if($score == -1) {
	    $score = 0;
	}
	# We're ranking things 1-5, so lower should win.
	$score = $score * -1;
	push(@scores, $score);
    }
    
    for($i = 0; $i < @systems; $i++) {
	for($j = $i+1; $j < @systems; $j++) {
	    $systemA = $systems[$i];
	    $systemB = $systems[$j];
	    
	    $judgmentA = $scores[$i];
	    $judgmentB = $scores[$j];
	    
	    # keep the systems and their scored alphabetically ordered
	    if($systemA gt $systemB) {
		$tmp = $systemA;
		$systemA = $systemB;
		$systemB = $tmp;
		    
		$tmp = $judgmentA;
		$judgmentA = $judgmentB;
		$judgmentB = $tmp;
	    }
	    

	    # if A is worse than B
	    if($judgmentA < $judgmentB) {
		$ranks{$langPair}{$testSet}{$segID}{$systemA}{$systemB}--;
		$counts{$langPair}{$testSet}{$segID}{$systemA}{$systemB}++;
	    } elsif ($judgmentB < $judgmentA) {
		$ranks{$langPair}{$testSet}{$segID}{$systemA}{$systemB}++;
		$counts{$langPair}{$testSet}{$segID}{$systemA}{$systemB}++;
	    }
	    # ccb - debugging
	    #print "$langPair\t$testSet\t$segID\t$systemA\t$systemB\t$judgmentA\t$judgmentB\n";
	}
    }   
}







open(SCORES, $ARGV[1]) or die("Couldn't open $ARGV[1]\n");
while($line = <SCORES>) {
    chomp $line;
    ($metric, $langPair, $testSet, $sysID, $segID, $score) = split(/\s+/, $line);
    if($errorMetrics{$metric} == 1) {
	$scores{$metric}{$langPair}{$testSet}{$sysID}{$segID} = ($score * -1);
    } else {
	$scores{$metric}{$langPair}{$testSet}{$sysID}{$segID} = $score;
    }
    $hasScore{$metric}{$langPair}{$testSet}{$sysID}{$segID} = 1;
}
close(SCORES);






# Count up the number of correct judgments...
foreach $langPair (keys %counts) {
    foreach $testSet (keys %{$counts{$langPair}}) {

	%numCorrect = ();
	%numIncorrect = ();

	foreach $segID (keys %{$counts{$langPair}{$testSet}}) {
	    foreach $systemA (keys %{$counts{$langPair}{$testSet}{$segID}}) {
		foreach $systemB (keys %{$counts{$langPair}{$testSet}{$segID}{$systemA}}) {
		    $count = $counts{$langPair}{$testSet}{$segID}{$systemA}{$systemB};
		    $rank = $ranks{$langPair}{$testSet}{$segID}{$systemA}{$systemB};
		    # pay attention only to those segments for which we have multiple judgments,
		    # and for which all annotators are unanimous...
#		    if(($count > 1 && ($count == $rank || $count == ($rank * -1))) 
#			|| ($analyzeAll == 1 && $count  == 1 && $rank != 0)) {
		    # ccb - debugging
		    if($rank != 0) {
			# ccb - debugging
			#print "$langPair $testSet $segID $systemA $systemB $rank $count\n";
			foreach $metric (@metrics) {
			    if($hasScore{$metric}{$langPair}{$testSet}{$systemA}{$segID} == 1 && 
			       $hasScore{$metric}{$langPair}{$testSet}{$systemB}{$segID}) {
				$scoreA = $scores{$metric}{$langPair}{$testSet}{$systemA}{$segID};
				$scoreB = $scores{$metric}{$langPair}{$testSet}{$systemB}{$segID};
				# ccb - debugging 
				#print "$langPair $testSet $segID $systemA $systemB $rank $metric $scoreA $scoreB\n";
				if(($rank > 0 && $scoreA > $scoreB) || ($rank < 0 && $scoreA < $scoreB)) {
				    $numCorrect{$metric}++;
				} else {
				    $numIncorrect{$metric}++;
				}
			    }
			}
		    }
		}
	    }
	}

	print "\n\n$langPair\t$testSet\n";
	foreach $metric (@metrics) {
	    $total = $numCorrect{$metric} + $numIncorrect{$metric};
	    if($total > 0) {
		$average = ($numCorrect{$metric} / $total);
		if($average =~ /\d\.\d\d\d/) {
		    $average =~ m/(\d\.\d\d)(\d)/;
		    $average = $1;
		    $roundingDigit = $2;
		    if($roundingDigit >= 5) {
			$average += 0.001;
		    }
		}
		$kendalls_tau = (($numCorrect{$metric} - $numIncorrect{$metric}) / $total);
		if($kendalls_tau =~ /\d\.\d\d\d/) {
		    $kendalls_tau =~ m/(\d\.\d\d)(\d)/;
		    $kendalls_tau = $1;
		    $roundingDigit = $2;
		    if($roundingDigit >= 5) {
			$kendalls_tau += 0.001;
		    }
		}
		print "$metric\t$numCorrect{$metric}\t$numIncorrect{$metric}\t$total\t$average\t$kendalls_tau\n";
	    }
	}



    }
}



