#!/usr/bin/perl
#
# This program first reads in the automatic metrics' scores.  It then
# reads in the human judgments and for each item, it checks the correspondence.


@testSets = ( "newssyscombtest2011" );
$testTypeToUse = "HUMAN_RANK";

# Foreign metrics
@metrics = ( "AMBER", "AMBER_nl", "AMBER_ti", "BLEU", "dfki_parseconf", "F15", "F15g3", "meteor-1.3-adq", "meteor-1.3-rank", "mp4ibm1", "mpF", "MTeRater", "MTeRater-Plus", "ROSE", "TER", "tesla-b", "tesla-f", "tesla-m", "TINE-srl_match", "wmpF" );
@langPairs = ( "en-fr", "en-de", "en-es", "en-cz");


# English metrics
@metrics = ( "AMBER", "AMBER_nl", "AMBER_ti", "BLEU", "dfki_parseconf", "F15", "F15g3", "meteor-1.3-adq", "meteor-1.3-rank", "mp4ibm1", "mpF", "MTeRater", "MTeRater-Plus", "ROSE", "TER", "tesla-b", "tesla-f", "tesla-m", "TINE-srl_match", "wmpF" );
#@langPairs = ("fr-en", "de-en", "es-en", "cz-en", "htraw-en", "htclean-en");
@langPairs = ("fr-en", "de-en", "es-en", "cz-en");



@errorMetrics = ();
$errorMetrics{"mp4ibm1"} = 1;
$errorMetrics{"MTeRater"} = 1;
$errorMetrics{"MTeRater-Plus"} = 1;
$errorMetrics{"TER"} = 1;


open(SCORES, $ARGV[0]) or die("Couldn't open $ARGV[0]\n");
while($line = <SCORES>) {
    chomp $line;
    ($metric, $langPair, $testSet, $sysID, $segID, $score) = split(/\s+/, $line);    
    if($errorMetrics{$metric} == 1) {
	$score = $score * -1;
    }
    $scores{$metric}{$langPair}{$testSet}{$sysID}{$segID} = $score;
    $hasScore{$metric}{$langPair}{$testSet}{$sysID}{$segID} = 1;
}
close(SCORES);

%numCorrect = ();


open(JUDGMENTS, $ARGV[1]) or die("Couldn't open $ARGV[1]\n");
$line = <JUDGMENTS>;

# FILE FORMAT FOR THE WMT11 human judgments
#($srclang, $trglang, $srcIndex, $documentId, $segmentId, $judgeId, $system1Number, $system1Id, $system2Number, $system2Id, $system3Number, $system3Id, $system4Number, $system4Id, $system5Number, $system5Id, $system1rank, $system2rank, $system3rank, $system4rank, $system5rank)
#  $srclang = $fields[0];
#  $trglang = $fields[1];
#  $srcIndex = $fields[2];
#  $documentId = $fields[3];
#  $segmentId = $fields[4];
#  $judgeId = $fields[5];
#  $system1Number = $fields[6];
#  $system1Id = $fields[7];
#  $system2Number = $fields[8];
#  $system2Id = $fields[9];
#  $system3Number = $fields[10];
#  $system3Id = $fields[11];
#  $system4Number = $fields[12];
#  $system4Id = $fields[13];
#  $system5Number = $fields[14];
#  $system5Id = $fields[15];
#  $system1rank = $fields[16];
#  $system2rank = $fields[17];
#  $system3rank = $fields[18];
#  $system4rank = $fields[19];
#  $system5rank = $fields[20];   

while($line = <JUDGMENTS>) {
    chomp $line;
    @fields = split(/,/, $line);
    $sourceLang = $fields[0];
    $targetLang = $fields[1];
    $segID = $fields[2]; 
    ($langPair, $testSet) = split(/\./, $fields[3]);
    $judgmentID = $fields[4];
    $annotatorID = $fields[5];
    $testSet = "newssyscombtest2011";
    $taskName = "WMT11 " . $sourceLang . "-" . $targetLang . " " . $testSet; 
    $testType = "HUMAN_RANK"; 

    @systems = ();
    for($i = 7; $i < 16; $i+=2) {
	$system = $fields[$i];
	push(@systems, $system);
    }

    @scores = ();
    for($i = 16; $i < @fields; $i++) {
	$score = $fields[$i];
	# -1 indicates no system.
	if($score == -1) {
	    $score = 0;
	}
	# We're ranking things 1-5, so lower should win.
	$score = $score * -1;
	push(@scores, $score);
    }
	
    for($i = 0; $i < @systems; $i++) {
	for($j = $i+1; $j < @systems; $j++) {
	    $systemA = $systems[$i];
	    $systemB = $systems[$j];
	    
	    $judgmentA = $scores[$i];
	    $judgmentB = $scores[$j];
		
	    $allMetricsHaveScoresForThis = 1;
	    foreach $metric (@metrics) {
		if($hasScore{$metric}{$langPair}{$testSet}{$systemA}{$segID} != 1) {
		    $allMetricsHaveScoresForThis = 0;
		}
		if($hasScore{$metric}{$langPair}{$testSet}{$systemB}{$segID} != 1) {
		    $allMetricsHaveScoresForThis = 0;
		}
	    }	    
		
		

	    #if($allMetricsHaveScoresForThis == 1) {
	    # ccb - allow/disallow ties
	    if($judgmentA != $judgmentB) {
		foreach $metric (@metrics) {
		    $scoreA = $scores{$metric}{$langPair}{$testSet}{$systemA}{$segID};
		    $scoreB = $scores{$metric}{$langPair}{$testSet}{$systemB}{$segID};
		    
		    if(($judgmentA > $judgmentB && $scoreA > $scoreB)
		       || ($judgmentA < $judgmentB && $scoreA < $scoreB)
		       || ($judgmentA == $judgmentB && $scoreA == $scoreB)) {
				$numCorrect{$metric}{$langPair}{$testSet}++;
		    } else {
				$numIncorrect{$metric}{$langPair}{$testSet}++;
		    }
		}
	    }
	}
    }
}



foreach $testSet (@testSets) {
    foreach $langPair (@langPairs) {
	foreach $metric (@metrics) {
	    $overallNumCorrect{$metric} += $numCorrect{$metric}{$langPair}{$testSet};
	    $overallNumIncorrect{$metric} += $numIncorrect{$metric}{$langPair}{$testSet};
	    
	    $total = $numCorrect{$metric}{$langPair}{$testSet} + $numIncorrect{$metric}{$langPair}{$testSet};
	    if($total > 0) {
		$average = ($numCorrect{$metric}{$langPair}{$testSet} / $total);
		if($average =~ /\d\.\d\d\d\d/) {
		    $average =~ m/(\d\.\d\d\d)(\d)/;
		    $average = $1;
		    $roundingDigit = $2;
		    if($roundingDigit >= 5) {
			$average += 0.001;
		    }
		}
		print "$testSet\t$langPair\t";
		print "$metric\t$numCorrect{$metric}{$langPair}{$testSet}\t$numIncorrect{$metric}{$langPair}{$testSet}\t$total\t" . ($average) . "\n";
	    }
	}
	print "\n";
    }
}


#overall
foreach $metric (@metrics) {
    $total = $overallNumCorrect{$metric} + $overallNumIncorrect{$metric};
    if($total > 0) {
	$average = ($overallNumCorrect{$metric} / $total);
	if($average =~ /\d\.\d\d\d\d/) {
	    $average =~ m/(\d\.\d\d\d)(\d)/;
	    $average = $1;
	    $roundingDigit = $2;
	    if($roundingDigit >= 5) {
		$average += 0.001;
	    }
	}
	print "$Overall\tOverall\t";

	print "$metric\t$overallNumCorrect{$metric}\t$overallNumIncorrect{$metric}\t$total\t" . ($average) . "\n";
    }
}
print "\n";
